﻿using System;
using System.Linq.Expressions;
using System.Windows.Forms;

namespace ExpressionTreeCalculator
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            string equation = EquationTextBox.Text;

            //remove white space
            equation.Replace(" ", "");

            Expression leftExpression = null;
            Expression rightExpression;
            Expression mathExpression;

            string currentDigit = string.Empty;
            char currentOperation = ' ';
            foreach (char c in equation)
            {
                if (Char.IsDigit(c))
                {
                    currentDigit += c;
                }
                else
                {
                    if (!String.IsNullOrEmpty(currentDigit))
                    {
                        if (leftExpression == null)
                        {
                            leftExpression = GetDigitConstant(currentDigit);
                        }
                        else
                        {
                            rightExpression = GetDigitConstant(currentDigit);
                            mathExpression =
                                GetMathExpression(currentOperation, leftExpression, rightExpression);
                            leftExpression = mathExpression;
                        }
                        currentDigit = string.Empty;
                    }
                    currentOperation = c;
                }
            }

            rightExpression = GetDigitConstant(currentDigit);
            mathExpression = GetMathExpression(currentOperation, leftExpression, rightExpression);

            Func<int> func = CompileExpression(mathExpression);
            InOrderResultLabel.Text = String.Format("In Order: {0}", func());

            var orderOfOperationsVisitor = new OrderOfOperationsVisitor();
            mathExpression = orderOfOperationsVisitor.Visit(mathExpression);
            while (orderOfOperationsVisitor.SwapOccurred)
            {
                orderOfOperationsVisitor = new OrderOfOperationsVisitor();
                mathExpression = orderOfOperationsVisitor.Visit(mathExpression);
            }

            func = CompileExpression(mathExpression);
            OrderOfOperationsLabel.Text = String.Format("Order Of Operations: {0}", func());
        }

        private Func<int> CompileExpression(Expression mathExpression)
        {
            Expression<Func<int>> result = Expression.Lambda<Func<int>>(mathExpression, null);
            return result.Compile();
        }

        public Expression GetDigitConstant(string digit)
        {
            int intValue;
            if (int.TryParse(digit, out intValue))
            {
                return Expression.Constant(intValue, typeof (int));
            }

            decimal decimalValue;
            if (Decimal.TryParse(digit, out decimalValue))
            {
                return Expression.Constant(decimalValue, typeof (Decimal));
            }
            return null;
        }

        public Expression GetMathExpression(char operation,
                                            Expression leftExpression, Expression rightExpression)
        {
            switch (operation)
            {
                case '*':
                    return Expression.MultiplyChecked(leftExpression, rightExpression);
                case '+':
                    return Expression.AddChecked(leftExpression, rightExpression);
                case '/':
                    return Expression.Divide(leftExpression, rightExpression);
                case '-':
                    return Expression.SubtractChecked(leftExpression, rightExpression);
            }

            return null;
        }
    }
}