﻿using System.Linq.Expressions;

namespace ExpressionTreeCalculator
{
    public class OrderOfOperationsVisitor : ExpressionVisitor
    {
        public bool SwapOccurred { get; private set; }

        protected override Expression VisitBinary(BinaryExpression b)
        {
            if ((b.Left is BinaryExpression) && IsChildOrderOfOperationsLower(b, b.Left))
            {
                SwapOccurred = true;
                return SwapLeft(b, (BinaryExpression) b.Left);
            }

            if ((b.Right is BinaryExpression) && IsChildOrderOfOperationsLower(b, b.Right))
            {
                SwapOccurred = true;
                return SwapRight(b, (BinaryExpression) b.Right);
            }

            return base.VisitBinary(b);
        }

        private bool IsChildOrderOfOperationsLower(Expression current, Expression child)
        {
            if (current.NodeType == ExpressionType.MultiplyChecked
                || current.NodeType == ExpressionType.Divide)
            {
                if (child.NodeType == ExpressionType.AddChecked ||
                    child.NodeType == ExpressionType.SubtractChecked)
                {
                    return true;
                }
            }
            return false;
        }

        private Expression SwapLeft(BinaryExpression expression, BinaryExpression left)
        {
            Expression right = Expression.MakeBinary(expression.NodeType, left.Right, expression.Right);
            return Expression.MakeBinary(left.NodeType, left.Left, right);
        }

        private Expression SwapRight(BinaryExpression expression, BinaryExpression right)
        {
            Expression left = Expression.MakeBinary(expression.NodeType, expression.Left, right.Left);
            return Expression.MakeBinary(right.NodeType, left, right.Right);
        }
    }
}